//+------------------------------------------------------------------+
//|                                                    MatrixNet.mq5 |
//|                                    Copyright (c) 2023, Marketeer |
//|                          https://www.mql5.com/ru/articles/12187/ |
//+------------------------------------------------------------------+
#property script_show_inputs

input int Epochs = 1000;
input double Accuracy = 0.001;
input double RandomNoise = 0.0;

#define BATCH_PROP
#include <MatrixNet.mqh>

//+------------------------------------------------------------------+
//| Artificial data generator                                        |
//+------------------------------------------------------------------+
bool CreateData(matrix &data, matrix &target, const int count) 
{ 
   if(!data.Init(count, 3) || !target.Init(count, 1)) 
      return false; 
   data.Random(-10, 10);
   vector X1 = MathPow(data.Col(0) + data.Col(1) + data.Col(2), 2); 
   vector X2 = MathPow(data.Col(0), 2) + MathPow(data.Col(1), 2) + MathPow(data.Col(2), 2); 
   if(!target.Col(X1 / X2 / 3.0, 0)) 
      return false;
   if(RandomNoise > 0)
   {
      matrix noise;
      noise.Init(count, 3);
      noise.Random(0, RandomNoise);
      data += noise - RandomNoise / 2;
      
      noise.Resize(count, 1);
      noise.Random(-RandomNoise / 2, RandomNoise / 2);
      target += noise;
   }
   return true; 
} 

//+------------------------------------------------------------------+
//| Main function                                                    |
//+------------------------------------------------------------------+
void OnStart()
{
   const int layers[] = {3, 11, 7, 1};
   MatrixNetVisual net(layers);
   matrix data, target;
   CreateData(data, target, 100);
   matrix valid, test;
   CreateData(valid, test, 25);
   
   // NB: in practice you should normalize and clean up data from outliers
   // before training (here we generate artificially ideal data)
   
   Print("Training result: ", net.train(data, target, valid, test, Epochs, Accuracy));
   //Print(net.train(data, target, 10000, 0.00001));
   matrix w[];
   if(net.getBestWeights(w))
   {
      // for(int i = 0; i < ArraySize(w); ++i) Print(w[i]); // debug
      MatrixNet net2(w);
      if(net2.isReady())
      {
         Print("Best copy on training data: ", net2.test(data, target));
         Print("Best copy on validation data: ", net2.test(valid, test));
      }
   }
   
   while(!IsStopped())
   {
      Sleep(1000);
   }
}
//+------------------------------------------------------------------+
